<?php

namespace Drupal\commerce_shepa\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

class PaymentOffsiteForm extends BasePaymentOffsiteForm
{

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    $order = $payment->getOrder();
    $order_id = $order->id();
    $payer_email = $order->getEmail();
    $redirect = Url::fromUri('base:/checkout/' . $order_id . '/payment/return/', ['absolute' => TRUE])
      ->toString();
    $amount = (int)$payment->getAmount()->getNumber();
    if ($payment->getAmount()->getCurrencyCode() !== 'IRR') {
      // Converts Iranian Toman to Rials,
      // because shepa.com payment service accepts IRR
      $amount = $amount * 10;
    }
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $gateway_configuration = $payment_gateway_plugin->getConfiguration();
    $api_key = $gateway_configuration['api_key'];
    $url = 'https://merchant.shepa.com/api/v1/token';

    if ($gateway_configuration["mode"] == "test") {
      $api_key = 'sandbox';
      $url = 'https://sandbox.shepa.com/api/v1/token';
    }

    try {

      $params = [
        "api" => $api_key,
        "amount" => $amount,
        "callback" => $redirect,
        "description" => "$order_id",
      ];

      if(isset($payer_email) && $payer_email !="" )  $params['email'] = $payer_email;
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Content-Type: application/json',
      ]);
      $res = curl_exec($ch);
      curl_close($ch);
      $result = json_decode($res);

      if (!empty($result->success)) {
        $redirect_url = $result->result->url;
        $response = new RedirectResponse($redirect_url);
        $response->send();
        return;

      } else {
        drupal_set_message('shepa.com: error: ' . implode("<br>",$result->error), 'error');
      }

    } catch (\Exception $e) {
      drupal_set_message('EError: ' . $e->getMessage(), 'error');
    }

    $checkout_review_page = Url::fromUri('base:/checkout/' . $order_id . '/review', ['absolute' => TRUE])
      ->toString();
    return $this->buildRedirectForm($form, $form_state, $checkout_review_page, [], NULL);

  }
}
